(function() {
  if (window.UNJSBridge != null) {
      return;
  }

  var bridges = [
    'App.getLocale',
    'App.setLocale',
    'App.getAppUA',
    'UNWebView.closeLoading',
    'UNWebView.close',
    'Game.getGASToken',
    'Game.getUserInfo',
    'Game.getJoinMode',
    'Game.getMatchedUsers',
    'Game.uploadRecord',
    'Game.callShare',
    'Game.checkVersion',
    'Game.getHighestRecord',
    'Game.checkVersion',
    'Game.changeState',
    'UI.alert',
    'UI.confirm',
    'UI.toast',
    'IndieGame.gotoRate',
    'IndieGame.gotoUneedGameCenter',
    'IndieGame.preparePlay',
    'IndieGame.editNickname',
     'IndieGame.editUserAvatar',

  ];

  var idSeed = 1;
  var handles = {};

  function genHandleId() {
    return 'BridgeHandle_' + (idSeed++);
  }

  var isAndroid;
  var isLowLevelBridge;
  var ua = window.navigator.userAgent;
  if(window.androidBridge || /UNWebView\/android/.test(ua)) {
    isAndroid = true;
    isLowLevelBridge = /UNWebView\/android\/lowLevelBridge/.test(ua);
  }

  var JSBridge = {
    onNativeMessage: function(handleId, data) {
      var callback = handles[handleId];
      callback && callback(data);
      delete handles[handleId];
    },

    callNative: function(data, callback) {
      var handleId = genHandleId();
      if(callback) {
        handles[handleId] = callback;
      }
      if(isAndroid) {
        var messageStr = JSON.stringify({
          handleId: handleId,
          data: data
        });
        if(isLowLevelBridge) {
          window.prompt("unbridge://" + messageStr);
        } else {
          window.androidBridge.exec(messageStr);
        }
      } else {
        window.webkit.messageHandlers.defaultHandler.postMessage({
          handleId: handleId,
          data: data
        });
      }
    }

  };

  var UNJSBridge = window.UNJSBridge = {
    dispatchDocumentEvent: function(name) {
      var ev = document.createEvent('Events');
        ev.initEvent(name, true, true);
        document.dispatchEvent(ev);
    },
    onNativeMessage: JSBridge.onNativeMessage,
    callNative: JSBridge.callNative,
    strict: {}
  };

  bridges.forEach(function(bridge) {
    var bridgeSplit = bridge.split('.');
    var moduleName = bridgeSplit[0];
    var methodName = bridgeSplit[1];

    var moduleObj = UNJSBridge[moduleName] = UNJSBridge[moduleName] || {};

    moduleObj[methodName] = function(args, callback) {
      if(typeof args === 'function') {
        callback = args
        args = null
      }
      JSBridge.callNative({
        module: moduleName,
        method: methodName,
        args: args || {}
      }, callback);
    }
  });

  UNJSBridge.dispatchDocumentEvent("webview.jsbridge.ready")
})();
